#include <nds.h>
#include "fw.h"
#include "ipc.h"

void interrupthandler() {
	u32 flags=REG_IF&REG_IE;
	VBLANK_INTR_WAIT_FLAGS|=flags;
	REG_IF=flags;
}

//change power management register
void writePM(int flags) {
	call3 spi_write=(call3)0x2369;
	spi_write(SPI_DEVICE_POWER,0,1);
	spi_write(SPI_DEVICE_POWER,flags,0);
}

//shut down, wait for lid to open
void arm7sleep() {
	u32 ie_save;
	writePM(PM_TOPLIGHT | PM_BOTTOMLIGHT | PM_SOUND | PM_LED_SLOW);
	swiChangeSoundBias(0,0x400);
	ie_save=REG_IE;
	REG_IE=IRQ_LID;
	swiSleep();
	REG_IF=~0;
	REG_IE=ie_save;
	writePM(PM_TOPLIGHT | PM_BOTTOMLIGHT | PM_SOUND);
	swiChangeSoundBias(1,0x400);
}

static void clearmem(u32 dest, u32 size) {
	int i=0;
	swiFastCopy(&i, (u32*)dest, 0x01000000 | (size>>2));
}

void hardware_init() {
	REG_IME=REG_IE=0;
	
	writePM(PM_TOPLIGHT | PM_BOTTOMLIGHT | PM_SOUND);
	REG_POWERCNT=POWER_SOUND;
	swiChangeSoundBias(1,0x400);

	//reset DMA
	clearmem(0x40000B0, 0x30);

	//((call0)0x119d)();
}

int main() {
	u32 i;

	IPC_ARM9=-1;
	IPC_XKEYS=0;
	hardware_init();

	REG_DISPSTAT=DISP_VBLANK_IRQ;
	IRQ_HANDLER=interrupthandler;
	REG_IE=IRQ_VBLANK;
	REG_IF=~0;
	REG_IME=IME_ENABLE;

	load_PersonalData();
	
	while(1) {
		swiWaitForVBlank();

		IPC_XKEYS=REG_KEYXY;
		
		i=IPC_ARM9;
		if(i!=-1) {
			IPC_ARM9=-1;
			switch(i) {
			
				case IPC_NDS:
					load_card();
					hardware_init();
					IPC_ARM7=IPC_NDS_GO;
					(*(call0*)0x27ffe34)();
					break;
				case IPC_PASSME:
					hardware_init();
					((call0)0x80000c0)();
					break;
				case IPC_GBA:
					if(PersonalData->_user_data.gbaScreen)
						i=PM_BOTTOMLIGHT | PM_SOUND;
					else
						i=PM_TOPLIGHT | PM_SOUND;
					writePM(i);
					swiChangeSoundBias(0,0x400);
					((call0)0x11a4)();
					break;
				case IPC_SLEEP:
					arm7sleep();
					break;
				case IPC_POWEROFF:
					writePM(PM_OFF);
					break;
			}
		}
	}
	return 0;
}

