#include <nds.h>
#include "fw.h"

u8 writereadSPI(u8 data) {
	while (REG_SPICNT & SPI_BUSY);
	REG_SPIDATA = data;
	while (REG_SPICNT & SPI_BUSY);
	return REG_SPIDATA;
}

//write one page (up to 256 bytes) of nvram
void write_nvram(void *src, u32 dst, int size) {
	int i;

	//write enable
	REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
	writereadSPI(6);
	REG_SPICNT = 0;

	//Wait for Write Enable Latch
	REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
	writereadSPI(5);
	while((writereadSPI(0)&0x02)==0);
	REG_SPICNT = 0;

	//page write
	REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
	writereadSPI(0x0A);
	writereadSPI((dst&0xff0000)>>16);
	writereadSPI((dst&0xff00)>>8);
	writereadSPI(0);
	for (i=0; i<size; i++) {
		writereadSPI(((u8*)src)[i]);
	}

	REG_SPICNT = 0;
	// wait programming to finish
	REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
	writereadSPI(0x05);
	while(writereadSPI(0)&0x01);	//Write In Progress
	REG_SPICNT = 0;
}

//size must be a multiple of 4
void read_nvram(u32 src, void *dst, u32 size) {
	((call3)0x2437)(src,(u32)dst,size);
}

//read firmware settings
void load_PersonalData() {
	u32 src, count0, count1;

	read_nvram(0x20, &src, 4);		//find settings area
	src=(src&0xffff)*8;

	read_nvram(src+0x70, &count0, 4);	//pick recent copy
	read_nvram(src+0x170, &count1, 4);
	if((u16)count0<(u16)count1)
		src+=0x100;
		
	read_nvram(src, PersonalData, 0x80);
	if(swiCRC16(0xffff,PersonalData,0x70) != ((u16*)PersonalData)[0x72/2]) 	//invalid / corrupt?
		read_nvram(src^0x100, PersonalData, 0x80);	//try the older copy
}

//write firmware settings
void save_PersonalData() {
	u32 dst, count0, count1;

	read_nvram(0x20, &dst, 4);		//find settings area
	dst=(dst&0xffff)*8;

	read_nvram(dst+0x70, &count0, 4);	//pick older copy to replace
	read_nvram(dst+0x170, &count1, 4);
	if((u16)count0>(u16)count1) {
		dst+=0x100;
		count0=count0+1;
	} else {
		count0=count1+1;
	}
	((u16*)PersonalData)[0x70/2]=count0;
	((u16*)PersonalData)[0x72/2]=swiCRC16(0xffff,PersonalData,0x70);
	write_nvram(PersonalData, dst, 0x80);
}
