#include <nds.h>
#include "fw.h"
#include "../../arm7/include/ipc.h"
#include "font_bin.h"
#include "fontpal_bin.h"

void interrupthandler() {
	u32 flags=REG_IF&REG_IE;
	VBLANK_INTR_WAIT_FLAGS|=flags;
	REG_IF=flags;
}

inline void clearmem(u32 dest, u32 size) {
	int i=0;
	swiFastCopy(&i, (u32*)dest, 0x01000000 | (size>>2));
}

inline void copy(void *src, void *dst, int size) {
	swiFastCopy(src,dst,size>>2);
}

void gfx_init() {
	REG_POWERCNT=POWER_SWAP_LCDS | POWER_ALL_2D;
	vramSetMainBanks(VRAM_A_MAIN_BG,0,0,0);
	DISPLAY_CR=MODE_0_2D|DISPLAY_BG0_ACTIVE;
	SUB_DISPLAY_CR=MODE_0_2D;
	BG0_CR=BG_TILE_BASE(1);
	BG0_X0=0;
	BG0_Y0=0;
	swiFastCopy(&font_bin,(u16*)(CHAR_BASE_BLOCK(1)+32*32),font_bin_size/4);
	swiFastCopy(&fontpal_bin,(u16*)BG_PALETTE, fontpal_bin_size/4);
}

void hardware_init() {

	REG_IME=0;
	REG_IE=0;
	REG_DISPSTAT=0;
	DISPLAY_CR=0;
	SUB_DISPLAY_CR=0;

	//reset DMA
	clearmem(0x40000B0, 0x30);

	//clear VRAM
	REG_POWERCNT=POWER_LCD;
	VRAM_CR=0x80808080;
	VRAM_E_CR=0x80;
	VRAM_F_CR=0x80;
	VRAM_G_CR=0x80;
	VRAM_H_CR=0x80;
	VRAM_I_CR=0x80;
	clearmem(0x6800000,0xA4000);
	VRAM_CR=0;
	VRAM_E_CR=0;
	VRAM_F_CR=0;
	VRAM_G_CR=0;
	VRAM_H_CR=0;
	VRAM_I_CR=0;

	REG_POWERCNT=POWER_SWAP_LCDS | POWER_ALL_2D;
	clearmem(0x5000000, 0x800);	//palette
	clearmem(0x7000000, 0x800);	//OAM

	//swiSetHaltCR(1);
}

void arm9sleep() {
	IPC_ARM9=IPC_SLEEP;
	while(IPC_XKEYS & 0x80)
		swiWaitForVBlank();
}

u32 keyhit() {
	static u32 oldkey=~0;
	u32 k,x;
	k=(~REG_KEYINPUT)&0x3ff;
	x=(oldkey^k)&k;
	oldkey=k;
	return x;
}

char *text[]={"Boot SLOT-1","Boot SLOT-2 (NDS mode)","Boot SLOT-2 (GBA mode)","Power off"};
#define OPTS (sizeof(text)/sizeof(char*))
void option(int line, int select) {
	u16 *p=(u16*)(SCREEN_BASE_BLOCK(0)+line*64);
	char *s=text[line];
	*p++ = select? '*' : ' ';
	while(*s) *p++=*s++;
}

int main() {
	u32 i,selected;
	u32 k;

	swiDelay(0x200000);		//bigger delay reduces sound pop

	hardware_init();	
	gfx_init();

	REG_DISPSTAT=DISP_VBLANK_IRQ;
	IRQ_HANDLER=interrupthandler;
	REG_IE=IRQ_VBLANK;
	REG_IF=~0;
	REG_IME=IME_ENABLE;

	IPC_ARM7=-1;
	selected=0;
	while(1) {
		swiWaitForVBlank();
		for(i=0;i<OPTS;i++)
			option(i,i==selected);

		k=keyhit();
		if(IPC_XKEYS & 0x80)	//lid closed
			arm9sleep();
		if(k&KEY_START)
			memdump();
		if(k&KEY_UP)
			selected--;
		if(k&KEY_DOWN)
			selected++;
		selected%=OPTS;
		if(k&(KEY_A|KEY_B)) {
			switch(selected) {

				case 0:		//NDS
					IPC_ARM9=IPC_NDS;
					hardware_init();
					while(IPC_ARM7!=IPC_NDS_GO)
						swiDelay(0);
					(*(call0*)0x27ffe24)();
					break;
				case 1:		//passme
					hardware_init();
					IPC_ARM9=IPC_PASSME;
					*(u32*)0x27ffe24=0x027ffe28;
					*(u32*)0x27ffe28=0xe51ff00c;
					((call0)0x27ffe28)();
					break;
				case 2:		//GBA
					hardware_init();
					if(PersonalData->_user_data.gbaScreen)
						REG_POWERCNT=POWER_LCD;
					else
						REG_POWERCNT=POWER_SWAP_LCDS | POWER_LCD;
					IPC_ARM9=IPC_GBA;
					break;
				case 3:		//off
					IPC_ARM9=IPC_POWEROFF;
					break;
			}
		}
	}
	return 0;
}
