#include <nds.h>
#include <nds/arm9/input.h>

typedef void(*call3)(u32,void*,u32);
void read_nvram(u32 src, void *dst, u32 size) {
	((call3)0x2437)(src,dst,size);
}

u8 writeread(u8 data) {
	while (REG_SPICNT & SPI_BUSY);
	REG_SPIDATA = data;
	while (REG_SPICNT & SPI_BUSY);
	return REG_SPIDATA;
}

int flash_verify(u8 *src,u32 dst) {
	int i;
	u8 tmp[256];

	//write enable
	REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
	writeread(6);
	REG_SPICNT = 0;
	
	//Wait for Write Enable Latch to be set
	REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
	writeread(5);
	while((writeread(0)&0x02)==0); //Write Enable Latch
	REG_SPICNT = 0;
	
	//page write
	REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
	writeread(0x0A);
	writeread((dst&0xff0000)>>16);
	writeread((dst&0xff00)>>8);
	writeread(0);
	for (i=0; i<256; i++) {
		writeread(src[i]);
	}

	REG_SPICNT = 0;
	// wait programming to finish
	REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
	writeread(0x05);
	while(writeread(0)&0x01);	//Write In Progress
	REG_SPICNT = 0;

	//read back & compare
	read_nvram(dst,tmp,256);
	for(i=0;i<256;i++) {
		if(tmp[i]!=src[i])
			return 0;
	}
	return 1;
}

#define SCREENBUFFER 0x2300000
void text(int line, char *s) {
	u16 *p=(u16*)(SCREENBUFFER+line*64);
	while(*s) *p++=*s++;
}

#undef IPC
#define IPC ((u32*)0x27ff000)
int main () {
	u32 key;
	int i, size;
	u8 *fw=(u8*)0x2200000;
	u16 *progress;
	
	//get firmware header for arm9
	read_nvram(0, fw, 0x200);

	//wait for arm9
	IPC[0] = 0xFFFFFF & *(u32*)&fw[8];	//pass firmware id ("MAC?") to arm9
	while(IPC[1]!=IPC[0])
		swiDelay(0x10000);

	text(0,"Get ready to short SL1.");
	text(1,"Hold L+R+Start to start.");

	do {
		swiDelay(0x10000);
		key=(~REG_KEYINPUT)&0x3ff;
	} while(key!=(KEY_L|KEY_R|KEY_START));

	text(0,"Short the SL1 contact now!");
	text(1,"Don't turn off until you're toldwhen it's safe!");

	//show progress...
	size=(IPC[2]+0xff)&~0xff;
	size=(size+IPC[4]+0xff)&~0xff;
	progress=(u16*)(SCREENBUFFER+3*64);
	for(i=0;i<size;i+=256)
		*progress++='*';
	progress=(u16*)(SCREENBUFFER+3*64);
		
	//flash bootstrap (protected)
	size=IPC[2];
	for(i=0;i<size;) {
		if(flash_verify(fw+i,i)) {
			i+=0x100;
			*progress++=' ';
		} else
			swiDelay(0x100000);
	}

	//flash everything else
	size=IPC[4];
	fw=(u8*)IPC[3];
	for(i=0;i<size;) {
		if(flash_verify(fw+i,i+0x10000)) {
			i+=0x100;
			*progress++=' ';
		} else
			swiDelay(0x100000);
	}

	text(1,"Done! You can turn off your DS.                ");
	while(1);
}
