#include <nds.h>
#include "font_bin.h"
#include "fontpal_bin.h"
#include "firmware_nds.h"

#define SCREENBUFFER 0x2700000		//arm7 draws stuff here

int makebootstrap(u8 *firmware);

void interrupthandler() {
	u32 flags=REG_IF&REG_IE;
	swiFastCopy((u32*)SCREENBUFFER,(u32*)SCREEN_BASE_BLOCK(0),0x200);
	VBLANK_INTR_WAIT_FLAGS|=flags;
	REG_IF=flags;
}

void gfx_init() {
	int i;
	REG_POWERCNT=POWER_SWAP_LCDS | POWER_LCD | POWER_2D_A;
	vramSetMainBanks(VRAM_A_MAIN_BG,0,0,0);
	DISPLAY_CR=MODE_0_2D|DISPLAY_BG0_ACTIVE;
	SUB_DISPLAY_CR=MODE_0_2D;
	BG0_CR=BG_TILE_BASE(1);
	BG0_X0=0;
	BG0_Y0=0;
	swiFastCopy(&font_bin,(u16*)(CHAR_BASE_BLOCK(1)+32*32),font_bin_size/4);
	swiFastCopy(&fontpal_bin,(u16*)BG_PALETTE, fontpal_bin_size/4);
	i=0x20202020;
	swiFastCopy(&i,(u8*)SCREENBUFFER,0x01000200);
}

void error(char *s) {
	u16 *p=(u16*)(SCREENBUFFER);
	while(*s) *p++=*s++;
	while(1) swiWaitForVBlank();
}

#undef IPC
#define IPC ((u32*)0x27ff000)
int main(void) {
	
	REG_DISPSTAT=DISP_VBLANK_IRQ;
	IRQ_HANDLER=interrupthandler;
	REG_IE=IRQ_VBLANK;
	REG_IF=~0;
	REG_IME=IME_ENABLE;

	gfx_init();

	//wait for arm7 to read old firmware
	while(IPC[0] != 0x43414d)
		swiWaitForVBlank();

	//prepare bootstrap area
	IPC[2]=makebootstrap((u8*)0x2600000);

	//everything else
	if(firmware_nds_size>0x30000-0x800) error("Firmware too big");
	IPC[3]=(u32)firmware_nds;
	IPC[4]=firmware_nds_size;

	//tell arm7 we're done
	IPC[1]=IPC[0];
	while(1) swiWaitForVBlank();
}
