#include <nds.h>
#include <string.h>
#include "boot9_bin.h"
#include "boot7_bin.h"
#include "key.h"

int CompressLZ77(const unsigned char *InBuf, unsigned char *OutBuf, int InSize);

u32 lookup(u32 v) {
	u32 a,b,c,d;
	a=(v>>24);
	b=(v>>16)&0xff;
	c=(v>>8)&0xff;
	d=v&0xff;
	a=key[a+18];
	b=key[b+274];
	c=key[c+530];
	d=key[d+786];
	return d+(c^(b+a));
}

void encrypt(u32 *dst,int size) {
	int i;
	u32 a,b,c;
	while(size>0) {
		a=dst[1];
		b=dst[0];
		for(i=0;i<16;i++) {
			c=key[i]^a;
			a=b^lookup(c);
			b=c;
		}
		dst[0]=a^key[16];
		dst[1]=b^key[17];
		dst+=2;
		size-=8;
	}
}

#define BOOT9ROMADDR 0x180
#define BOOT9RAMADDR 0x23a0000
#define BOOT7RAMADDR 0x380f800

// prepare bootstrap area, returns size
// old header is in *firmware
int makebootstrap(u8 *firmware) {

	u8 *dst;
	u16 *fw16;
	u16 crc;
	int lz7size,lz9size;

	memset(firmware+BOOT9ROMADDR,0,0x10000);
	
//compress & encrypt

	dst=firmware+BOOT9ROMADDR;
	lz9size=CompressLZ77(boot9_bin, dst, boot9_bin_size);
	lz9size=(lz9size+7)&~7;
	encrypt((u32*)dst,lz9size);
	dst+=lz9size;
	
	lz7size=CompressLZ77(boot7_bin, dst, boot7_bin_size);
	lz7size=(lz7size+7)&~7;
	encrypt((u32*)dst,lz7size);
	dst+=lz7size;

//set header info

	fw16=(u16*)firmware;
	fw16[0x17c/2]=3;				//firmware ID:  1=old flashme (wifi unsafe), 2=flashme, 3=HB
	if(fw16[0x17e/2]==0xffff)			//preserve old crc (identifies original firmware version)
		fw16[0x17e/2]=fw16[0x06/2];

	fw16[0x08/2]=0x414d;				//"MA"
	fw16[0x0a/2]=0x5043;				//"CP"
	fw16[0x0c/2]=BOOT9ROMADDR>>3;			//arm9 src
	fw16[0x0e/2]=(0x2800000-BOOT9RAMADDR)>>8;	//arm9 dst
	fw16[0x10/2]=(BOOT9ROMADDR+lz9size)>>3;	//arm7 src
	fw16[0x12/2]=(0x3810000-BOOT7RAMADDR)>>8;	//arm7 dst
	fw16[0x14/2]=(1) | (6<<3) | (1<<6) | (6<<9);

	crc=swiCRC16(0xffff,(u8*)boot9_bin,boot9_bin_size);
	crc=swiCRC16(crc,(u8*)boot7_bin,boot7_bin_size);
	fw16[0x06/2]=crc;		//new CRC

	return (int)dst-(int)firmware;
}
